<?php
namespace App\Controller;

use App\Exception\SqlException as SqlException;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebhookController {  

    const MAX_PARALLEL_ORDERS   = 240;
    const MAX_PARALLEL_TOKENS   = 100;
    const MAX_PARALLEL_DISPATCH = 10; 

    private $connection;
    private $webhookService;
    private $util;

    public function __construct(\Doctrine\DBAL\Connection $connection,
                                \App\Service\WebhookService $webhookService,
                                \App\Util\Util $util) {

        $this->connection       = $connection;
        $this->webhookService   = $webhookService;
        $this->util             = $util;
    }

    public function welcome(Request $request, Response $response) {
        
        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
        ]);
    }

    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    public function getCurrentAuthToken(Request $request, Response $response) {
        try {
            $authToken = $this->webhookService->getAuthToken();
    
            return $response->withJson([
                'authToken' => $authToken['token']
            ]);
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function getMerchantStatus(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $merchantId = $req['merchantId'];

        $this->authToken = $this->webhookService->getAuthToken();

        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $merchantStatus = json_decode($this->webhookService->getMerchantStatus($merchantId, $this->authToken['token']), true);

        return $response->withJson([
            'merchantStatus' => $merchantStatus
        ]);
    }
    
    public function getMerchantInterruptions(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $merchantId = $req['merchantId'];

        $this->authToken = $this->webhookService->getAuthToken();

        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $merchantInterruptions = json_decode($this->webhookService->getMerchantInterruptions($merchantId, $this->authToken['token']), true);

        return $response->withJson([
            'merchantInterruption' => $merchantInterruptions
        ]);
    }
    
    public function requestCancelOrder(Request $request, Response $response) {
        $req = $request->getParsedBody();

        $orderId = $req['orderId'];

        $this->authToken = $this->webhookService->getAuthToken();

        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $cancelStatus = $this->webhookService->requestCancelOrder($orderId, $this->authToken['token']);
        
        if($cancelStatus !== 202) {
            $error = true;
            $message = 'Não foi possível solicitar o cancelamento do pedido';
        } else {
            $error = false;
            $message = 'Solicitação de cancelamento realizada com sucesso para a plataforma do iFood';
        }

        return $response->withJson([
            'Error' => $error,
            'Message' => $message
        ]);
    }
    
    public function getActiveMerchants(Request $request, Response $response) {
        try{
            $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
            $configData = json_decode($configFile, true);
            $basePath   = $configData['basePath'];
            
            $this->authToken = $this->webhookService->getAuthToken();
            
            if(empty($this->authToken)) {
                $response->withStatus(200);
                return 0;
            }
            
            $url = 'https://merchant-api.ifood.com.br/merchant/v1.0/merchants?page=';
            
            $header = array(
                "Content-Type: application/json",
                "Authorization: bearer ".$this->authToken['token']
            );
            
            $merchantsArray = [];
            
            $activeMerchants = [];
            
            $i = 1;
            
            do {
                    $ch = curl_init();
                    
                    curl_setopt_array($ch, array(
                        CURLOPT_URL => $url.$i,
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING => '',
                        CURLOPT_MAXREDIRS => 10,
                        CURLOPT_TIMEOUT => 10,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
                        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST => 'GET',
                        CURLOPT_HTTPHEADER => $header
                    ));
                    
                    $info = curl_exec($ch);
                    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
                    
                    if (($httpCode !== 200 && $httpCode !== 201 && $httpCode !== 202 && $httpCode !== 204) || (stripos($info, "<html>"))){
                        
                        $fileName = date("H-i-s") . ".txt";
                        $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                            'eventTime' => date("d/m/Y H:i:s"),
                            'header'    => $header,
                            'url'       => $url.$i,
                            'method'    => "GET",
                            'params'    => [],
                            'httpCode'  => $httpCode,
                            'error'     => curl_error($ch),
                            'response'  => $info
                        ]));
                    }
                    
                    curl_close($ch);
                    
                    $activeMerchants = json_decode($info, true);
                    
                    if(!empty($activeMerchants)){
                        foreach($activeMerchants as $activeMerchant){
                            array_push($merchantsArray, $activeMerchant['id']);
                        }
                    }
                    
                    $i++;
            } while(!empty($activeMerchants));
            
            if(!empty($merchantsArray)){
                file_put_contents($basePath.'/activeMerchants.json', json_encode($merchantsArray));   
            }
            
            $response->withStatus(200);
            return 0;
        } catch(\Exception $e){
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function runPollingService(Request $request, Response $response) {
        try {
            $req = $request->getParsedBody();
            
            $this->authToken = $this->webhookService->getAuthToken();
            
            if(empty($this->authToken)) {
                $response->withStatus(200);
                return 0;
            }
            
            $fullMerchantList = $this->webhookService->getAllTokens();
            $merchantCount = 0;
            $merchantList = '';
            $arrayMerchantList = [];
            
            foreach($fullMerchantList as $merchant) {
                $merchantList .= $merchant . ',';
                $merchantCount++;
                
                if($merchantCount >= self::MAX_PARALLEL_TOKENS){
                    array_push($arrayMerchantList, substr($merchantList, 0, -1));
                    
                    $merchantCount = 0;
                    $merchantList = '';
                }
            }
            
            if($merchantCount > 0){
                array_push($arrayMerchantList, substr($merchantList, 0, -1));
            }
            
            $this->webhookService->getEvents($this->authToken['token'], $arrayMerchantList);
            
            return $response->withJson([
                'success' => true,
                'message' => 'pooling realizado com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function runAckService(Request $request, Response $response) {
        try {
            $req = $request->getParsedBody();
            
            $this->authToken = $this->webhookService->getAuthToken();
            
            if(empty($this->authToken)) {
                $response->withStatus(200);
                return 0;
            }
            
            $this->webhookService->runPendingACKs($this->authToken['token']);   
            
            return $response->withJson([
                'success' => true,
                'message' => 'ack realizado com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e){
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelOrders(Request $request, Response $response) {
        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $processingErrors = array();

        $responseOrders = [];
        $running = null;
        $handles = [];
        $c = 0;

        $numberOfPendingOrders = 0;

        try {
            $orders = $this->util->readEventFiles("order");
            
            $numberOfPendingOrders = count($orders);

            if($numberOfPendingOrders > 0) {
                
                //IFOOD URL
                $url = 'https://merchant-api.ifood.com.br/order/v1.0/orders/';
                
                //TEKNISA SANDBOX API URL
                //$url = 'http://localhost/odhen-ifood-bench/src/public/index.php/order/v1.0/orders/';
                $header = array(
                    "Content-Type: application/json",
                    "Authorization: bearer ".$this->authToken['token']
                );

                foreach($orders as $order) {
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $url.$order['orderId'],
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'GET',
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);
                
                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    $requestInfo = curl_getinfo($handles[$i]);
                    $url = $requestInfo['url'];
                    $httpCode = $requestInfo['http_code'];
                    
                    if (($httpCode !== 200 && $httpCode !== 201 && $httpCode !== 202) || (stripos($info, "<html>"))){
                        if($httpCode === 0){
                            $info = 'Timeout was reached.';
                        }
                        
                        $requestData = [
                            'header'    => $header,
                            'url'       => $url,
                            'method'    => 'GET',
                            'params'    => [],
                            'httpCode'  => $httpCode,
                            'error'     => $info
                        ];
                        
                        $fileName = date("H-i-s") . "-" . round(microtime(true) * 1000) . ".txt";
                        
                        $this->util->writeLog('network', $fileName, $this->util->formatMultiNetworkError($requestData));
                    } else{
                        $orderData = json_decode($info, true);
                        if(!isset($orderData['error'])){
                            array_push($responseOrders, $orderData);   
                        }
                    }
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);


                $integratedOrders = [];
                $running = null;
                $handles = [];
                $c = 0;

                foreach($responseOrders as $responseOrder) {
                    $clientData = $this->webhookService->getClientData($responseOrder['merchant']['id']);
                    
                    $clientName = $clientData['NOME'];
                    $clientType = $clientData['TIPO'];
                    $dbName     = $clientData['DBNAME'];
                    
                    if($clientType == 'hosting' || $clientType == 'saas'){
                        $header = array(
                            "Content-type: application/json",
                            "x-client-name: $clientName",
                            "x-client-type: $clientType",
                            "x-client-env: $dbName"
                        );
                    } else{
                        $header = array(
                            "Content-Type: application/json"
                        );
                    }

                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $clientData['URL']."/start",
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 10,
                        CURLOPT_FOLLOWLOCATION => true,
                        CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'POST',
                        CURLOPT_POSTFIELDS     => json_encode([
                            "orderData" => $responseOrder,
                            "authToken" => $this->authToken['token']
                        ]),
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $headersArray[$c] = $header;
                    
                    $paramsArray[$c] = [
                        "orderData" => $responseOrder,
                        "authToken" => $this->authToken['token']
                    ];

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);

                for($i = 0; $i < $c; $i++) {
                    $info = curl_multi_getcontent($handles[$i]);
                    $requestInfo = curl_getinfo($handles[$i]);
                    $url = $requestInfo['url'];
                    $httpCode = $requestInfo['http_code'];
                    
                    if (($httpCode !== 200 && $httpCode !== 201 && $httpCode !== 202) || (stripos($info, "<html>"))) {
                        if($httpCode === 0) {
                            $info = 'Timeout was reached.';
                        }
                        
                        $requestData = [
                            'header'    => $headersArray[$i],
                            'url'       => $url,
                            'method'    => 'POST',
                            'params'    => $paramsArray[$i],
                            'httpCode'  => $httpCode,
                            'error'     => $info
                        ];
                        
                        $fileName = date("H-i-s") . "-" . round(microtime(true) * 1000) . ".txt";
                        
                        $this->util->writeLog('network', $fileName, $this->util->formatMultiNetworkError($requestData));
                    } else {
                        array_push($integratedOrders, json_decode($info, true));
                    }
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);

                foreach($responseOrders as $responseOrder) {
                    $fileName = $orders[$responseOrder['id']]['id'] . ".json";
                    $this->util->manipulateEventFile("processed", $fileName, json_encode($orders[$responseOrder['id']]));
                }
            
                return $response->withJson([
                    'success' => true,
                    'message' => 'orders realizado com sucesso',
                    'result'  => $integratedOrders
                ]);

            }
            
            return $response->withJson([
                'success' => true,
                'message' => 'orders realizado com sucesso',
                'result'  => []
            ]);
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelCancellationRequests(Request $request, Response $response) {
        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $processingErrors = array();

        $responseOrders = [];
        $running = null;
        $handles = [];
        $c = 0;

        $numberOfPendingCancelRequests = 0;

        try {
            $cancelRequests = $this->util->readEventFiles("cancelRequest");
            
            $numberOfPendingCancelRequests = count($cancelRequests);

            if($numberOfPendingCancelRequests > 0) {
                
                //IFOOD URL
                $url = 'https://merchant-api.ifood.com.br/order/v1.0/orders/';
                
                //TEKNISA SANDBOX API URL
                //$url = 'http://localhost/odhen-ifood-bench/src/public/index.php/order/v1.0/orders/';
                $header = array(
                    "Content-Type: application/json",
                    "Authorization: bearer ".$this->authToken['token']
                );

                foreach($cancelRequests as $cancelRequest) {
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $url.$cancelRequest['orderId'],
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'GET',
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);

                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    $requestInfo = curl_getinfo($handles[$i]);
                    $url = $requestInfo['url'];
                    $httpCode = $requestInfo['http_code'];
                    
                    if (($httpCode !== 200 && $httpCode !== 201 && $httpCode !== 202) || (stripos($info, "<html>"))){
                        if($httpCode === 0){
                            $info = 'Timeout was reached.';
                        }
                        
                        $requestData = [
                            'header'    => $header,
                            'url'       => $url,
                            'method'    => 'GET',
                            'params'    => [],
                            'httpCode'  => $httpCode,
                            'error'     => $info
                        ];
                        
                        $fileName = date("H-i-s") . "-" . round(microtime(true) * 1000) . ".txt";
                        
                        $this->util->writeLog('network', $fileName, $this->util->formatMultiNetworkError($requestData));
                    } else{
                        array_push($responseOrders, json_decode($info, true));
                    }
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);

                $requestCancelOrders = [];
                $running = null;
                $handles = [];
                $c = 0;

                foreach($responseOrders as $responseOrder) {
                    $clientData = $this->webhookService->getClientData($responseOrder['merchant']['id']);
                    
                    $clientName = $clientData['NOME'];
                    $clientType = $clientData['TIPO'];
                    $dbName     = $clientData['DBNAME'];
                    
                    if($clientType == 'hosting' || $clientType == 'saas'){
                        $header = array(
                            "Content-type: application/json",
                            "x-client-name: $clientName",
                            "x-client-type: $clientType",
                            "x-client-env: $dbName"
                        );
                    } else {
                        $header = array(
                            "Content-Type: application/json"
                        );
                    }
                    
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $clientData['URL']."/cancelOrderRequest",
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'POST',
                        CURLOPT_POSTFIELDS     => json_encode([
                            "orderData" => $responseOrder,
                            "authToken" => $this->authToken['token']
                        ]),
                        CURLOPT_HTTPHEADER     => $header
                    ));
                    
                    $headersArray[$c] = $header;
                    
                    $paramsArray[$c] = [
                        "orderData" => $responseOrder,
                        "authToken" => $this->authToken['token']
                    ];

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);
                
                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    $requestInfo = curl_getinfo($handles[$i]);
                    $url = $requestInfo['url'];
                    $httpCode = $requestInfo['http_code'];
                    
                    if (($httpCode !== 200 && $httpCode !== 201 && $httpCode !== 202) || (stripos($info, "<html>"))){
                        if($httpCode === 0){
                            $info = 'Timeout was reached.';
                        }
                        
                        $requestData = [
                            'header'    => $headersArray[$i],
                            'url'       => $url,
                            'method'    => 'POST',
                            'params'    => $paramsArray[$i],
                            'httpCode'  => $httpCode,
                            'error'     => $info
                        ];
                        
                        $fileName = date("H-i-s") . "-" . round(microtime(true) * 1000) . ".txt";
                        
                        $this->util->writeLog('network', $fileName, $this->util->formatMultiNetworkError($requestData));
                    } else{
                        array_push($requestCancelOrders, json_decode($info, true));
                    }
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }
                
                foreach($requestCancelOrders as $requestCancelOrder) {
                    if($requestCancelOrder['success'] === true) {
                        $fileName = $cancelRequests[$requestCancelOrder['orderId']]['id'] . ".json";
                        $this->util->manipulateEventFile("processed", $fileName, json_encode($cancelRequests[$requestCancelOrder['orderId']]));
                    }
                }

                curl_multi_close($mh);

                return $response->withJson([
                    'success' => true,
                    'message' => 'cancel requests realizado com sucesso',
                    'result'  => []
                ]);
            }
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelCancellations(Request $request, Response $response) {

        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }

        $processingErrors = array();

        $responseOrders = [];
        $running = null;
        $handles = [];
        $c = 0;

        $numberOfPendingCancellations = 0;

        try {
            $cancellations = $this->util->readEventFiles("cancelled");
            
            $numberOfPendingCancellations = count($cancellations);

            if($numberOfPendingCancellations > 0) {
                
                //IFOOD URL
                $url = 'https://merchant-api.ifood.com.br/order/v1.0/orders/';
                
                //TEKNISA SANDBOX API URL
                //$url = 'http://localhost/odhen-ifood-bench/src/public/index.php/order/v1.0/orders/';
                $header = array(
                    "Content-Type: application/json",
                    "Authorization: bearer " . $this->authToken['token']
                );

                foreach($cancellations as $cancellation) {
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $url.$cancellation['orderId'],
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'GET',
                        CURLOPT_HTTPHEADER     => $header
                    ));

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);

                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    $requestInfo = curl_getinfo($handles[$i]);
                    $url = $requestInfo['url'];
                    $httpCode = $requestInfo['http_code'];
                    
                    if (($httpCode !== 200 && $httpCode !== 201 && $httpCode !== 202) || (stripos($info, "<html>"))){
                        if($httpCode === 0){
                            $info = 'Timeout was reached.';
                        }
                        
                        $requestData = [
                            'header'    => $header,
                            'url'       => $url,
                            'method'    => 'GET',
                            'params'    => [],
                            'httpCode'  => $httpCode,
                            'error'     => $info
                        ];
                        
                        $fileName = date("H-i-s") . "-" . round(microtime(true) * 1000) . ".txt";
                        
                        $this->util->writeLog('network', $fileName, $this->util->formatMultiNetworkError($requestData));
                    } else{
                        array_push($responseOrders, json_decode($info, true));
                    }
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }

                curl_multi_close($mh);

                $cancelledOrders = [];
                $running = null;
                $handles = [];
                $c = 0;

                foreach($responseOrders as $responseOrder) {
                    $clientData = $this->webhookService->getClientData($responseOrder['merchant']['id']);
                    
                    $clientName = $clientData['NOME'];
                    $clientType = $clientData['TIPO'];
                    $dbName     = $clientData['DBNAME'];
                    
                    if($clientType == 'hosting' || $clientType == 'saas'){
                        $header = array(
                            "Content-type: application/json",
                            "x-client-name: $clientName",
                            "x-client-type: $clientType",
                            "x-client-env: $dbName"
                        );
                    } else{
                        $header = array(
                            "Content-Type: application/json"
                        );
                    }
                    
                    $handles[$c] = curl_init();

                    curl_setopt_array($handles[$c], array(
                        CURLOPT_URL            => $clientData['URL']."/cancelOrder",
                        CURLOPT_RETURNTRANSFER => true,
                        CURLOPT_ENCODING       => '',
                        CURLOPT_MAXREDIRS      => 10,
                        CURLOPT_TIMEOUT        => 45,
                        CURLOPT_CONNECTTIMEOUT => 5,
                        CURLOPT_FOLLOWLOCATION => true,
						CURLOPT_SSL_VERIFYPEER => false,
                        CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
                        CURLOPT_CUSTOMREQUEST  => 'POST',
                        CURLOPT_POSTFIELDS     => json_encode([
                            "orderData" => $responseOrder,
                            "authToken" => $this->authToken['token']
                        ]),
                        CURLOPT_HTTPHEADER     => $header
                    ));
                    
                    $headersArray[$c] = $header;
                    
                    $paramsArray[$c] = [
                        "orderData" => $responseOrder,
                        "authToken" => $this->authToken['token']
                    ];

                    $c++;

                    if($c >= self::MAX_PARALLEL_ORDERS) {
                        break;
                    }
                }

                $mh = curl_multi_init();
                for($i = 0; $i < $c; $i++) {
                    curl_multi_add_handle($mh, $handles[$i]);
                }
                    
                do {
                    curl_multi_exec($mh, $running);
                } while ($running > 0);
                
                for($i = 0; $i < $c; $i++){
                    $info = curl_multi_getcontent($handles[$i]);
                    $requestInfo = curl_getinfo($handles[$i]);
                    $url = $requestInfo['url'];
                    $httpCode = $requestInfo['http_code'];
                    
                    if (($httpCode !== 200 && $httpCode !== 201 && $httpCode !== 202) || (stripos($info, "<html>"))){
                        if($httpCode === 0){
                            $info = 'Timeout was reached.';
                        }
                        
                        $requestData = [
                            'header'    => $headersArray[$i],
                            'url'       => $url,
                            'method'    => 'POST',
                            'params'    => $paramsArray[$i],
                            'httpCode'  => $httpCode,
                            'error'     => $info
                        ];
                        
                        $fileName = date("H-i-s") . "-" . round(microtime(true) * 1000) . ".txt";
                        
                        $this->util->writeLog('network', $fileName, $this->util->formatMultiNetworkError($requestData));
                    } else{
                        array_push($cancelledOrders, json_decode($info, true));
                    }
                }

                for($i = 0; $i < $c; $i++) {
                    curl_multi_remove_handle($mh, $handles[$i]);
                }
                
                foreach($cancelledOrders as $cancelledOrder) {
                    if($cancelledOrder['success'] === true) {
                        $fileName = $cancellations[$cancelledOrder['orderId']]['id'] . ".json";
                        $this->util->manipulateEventFile("processed", $fileName, json_encode($cancellations[$cancelledOrder['orderId']]));
                    }
                }

                curl_multi_close($mh);
                
                return $response->withJson([
                    'success' => true,
                    'message' => 'cancellation realizado com sucesso',
                    'result'  => []
                ]);
            }
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function processParallelDispatch(Request $request, Response $response) {

        $req = $request->getParsedBody();
            
        $this->authToken = $this->webhookService->getAuthToken();
        
        if(empty($this->authToken)) {
            $response->withStatus(200);
            return 0;
        }
    
        try {
            $merchants = $this->webhookService->getMerchantsForDispatch();
            
            $numberOfMerchants = count($merchants);
            if($numberOfMerchants > 0) {
    
                // Divide o array em chunks de 20 posições (MAX_PARALLEL_DISPATCH)
                $chunks = array_chunk($merchants, self::MAX_PARALLEL_DISPATCH);
                
                // Para cada chunk, chama a função processDispatchChunk
                foreach ($chunks as $merchantChunk) {
                    $this->webhookService->processDispatchChunk($this->authToken['token'], $merchantChunk);
                    sleep(1);
                }
    
                return $response->withJson([
                    'success' => true,
                    'message' => 'dispatch realizado com sucesso',
                    'result'  => [
                        'numberOfMerchants' => $numberOfMerchants
                    ]
                ]);
            }
        } catch(\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function getClientList(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->getClientList()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function getWebhookParams(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->getWebhookParams()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function updateParams(Request $request, Response $response){
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->updateParams($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteClient(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Cliente deletado com sucesso!",
                'data'    => $this->webhookService->deleteClient($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function addToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->addToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function editToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "",
                'data'    => $this->webhookService->editToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function deleteToken(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            return $response->withJson([
                'success' => true,
                'message' => "Token deletado com sucesso!",
                'data'    => $this->webhookService->deleteToken($params)
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
    
    public function cleanToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token removido com sucesso!",
                'data'    => $this->webhookService->cleanToken()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }

    public function renovateToken(Request $request, Response $response) {
        try {
            return $response->withJson([
                'success' => true,
                'message' => "Token renovado com sucesso!",
                'data'    => $this->webhookService->getAuthTokens()
            ]);
        } catch (\Exception $e) {
            return $response->withJson([
                'success' => false,
                'message' => $e->getMessage(),
                'result'  => []
            ]); 
        }
    }
}